import shutil
import re
import os
from kpl_helper.base import get_config, _APIBase
from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
import uuid
import logging

HAN_SCRIPT_PAT = re.compile(
    r'[\u4E00-\u9FEF\u3400-\u4DB5\u20000-\u2A6D6\u2A700-\u2B734'
    r'\u2B740-\u2B81D\u2D820-\u2CEA1\u2CEB0-\u2EBE0]'
)
MB = 1024 * 1024


def is_archive_file(path):
    if os.path.isdir(path):
        return False
    if path.endswith(".tar") or path.endswith(".tar.gz") or path.endswith(".zip"):
        return True
    return False


def assert_symbol_link(path):
    path = os.path.abspath(path)
    if os.path.islink(path):
        raise Exception("[kpl-helper]: `{}` is symbol link. cannot make archive".format(path))


class _UploadProgress:
    def __init__(self, total, bar_size=50):
        if total < 1024:
            self.unit_divisor = 1
            self.unit = "B"
        elif total < MB:
            self.unit_divisor = 1024
            self.unit = "KB"
        else:
            self.unit_divisor = MB
            self.unit = "MB"
        self.total = int(total / self.unit_divisor)
        self.bar_size = bar_size

    def update(self, current):
        current = int(current / self.unit_divisor)
        progress = int(current * self.bar_size / self.total)
        completed = str(int(current * 100 / self.total)) + '%'
        print('Progress: [{} {}{}] {}/{}{}'.format(chr(9608) * progress, completed,
                                                   '.' * (self.bar_size - progress),
                                                   current, self.total, self.unit), end='\r', flush=True)

    def close(self):
        print("\n")


class Uploader(_APIBase):
    @staticmethod
    def _make_archive(root_dir, base_dir):
        tar_path = os.path.join("/tmp", uuid.uuid4().hex)
        shutil.make_archive(tar_path, "tar",
                            root_dir=root_dir,
                            base_dir=base_dir)
        return tar_path + ".tar"

    def _upload(self, route, name, desc, path, make_archive=False, size_limit=None):
        path = os.path.abspath(path)
        if os.path.islink(path):
            raise Exception("[kpl-helper]: `{}` is symbol link. cannot make archive".format(path))
        if not get_config().get_inner():
            return
        upload_file = path
        if make_archive:
            root_dir = os.path.abspath(os.path.join(path, ".."))
            upload_file = self._make_archive(root_dir, os.path.basename(path))

        file_size = os.path.getsize(upload_file)
        if size_limit is not None and file_size > size_limit:
            raise Exception("[kpl-helper]: file size `{}MB` exceeds limit of 50MB".format(size_limit / MB))
        bar = _UploadProgress(file_size)

        def upload_callback(m):
            bar.update(m.bytes_read)

        with open(upload_file, "rb") as fi:
            encoder = MultipartEncoder(
                fields={'name': name, 'description': desc,
                        'file': (os.path.basename(upload_file), fi, 'text/plain')}
            )
            monitor = MultipartEncoderMonitor(encoder, upload_callback)
            self.post(route, data=monitor, headers={'Content-Type': monitor.content_type})
        bar.close()
        if make_archive:
            os.remove(upload_file)

    def upload_dataset(self, name, desc, path):
        # 如果是单个文件，且是.tar/.tar.gz/.zip文件，则直接上传
        # 如果是单个文件，且非.tar/.tar.gz/.zip文件，则需要对文件打包上传
        # 如果是文件夹，则需要对目录打包上传
        self._upload("/dataset/upload", name, desc, path, not is_archive_file(path))

    def upload_dataset_fs(self, name, desc, path):
        # 如果是单个文件，且是.tar/.tar.gz/.zip文件，则直接上传
        # 如果是单个文件，且非.tar/.tar.gz/.zip文件，则需要对文件打包上传
        # 如果是文件夹，则需要对目录打包上传
        self._upload("/dataset_fs/upload", name, desc, path, not is_archive_file(path))

    def upload_model(self, name, desc, path):
        # 模型如果是单个文件则不需要打包，否则需要打包成.tar文件再上传
        self._upload("/model/upload", name, desc, path, os.path.isdir(path))

    def upload_algorithm(self, name, desc, path):
        self._upload("/algorithm/upload", name, desc, path, not is_archive_file(path), 50 * MB)


def save_dataset(path, name, description, as_serialized=False):
    try:
        uploader = Uploader()
        if as_serialized:
            uploader.upload_dataset(name, description, path)
        else:
            uploader.upload_dataset_fs(name, description, path)
    except Exception as e:
        logging.exception(e)


def save_model(path, name, description):
    try:
        uploader = Uploader()
        uploader.upload_model(name, description, path)
    except Exception as e:
        logging.exception(e)


def save_algorithm(path, name, description):
    try:
        uploader = Uploader()
        uploader.upload_algorithm(name, description, path)
    except Exception as e:
        logging.exception(e)



import argparse
from kpl_helper.save import save_model, save_dataset, save_algorithm
from kpl_helper.stop import kill


def _save_model(args):
    save_model(args.path, args.name, args.desc)


def _save_dataset(args):
    save_dataset(args.path, args.name, args.desc, args.as_serialized)


def _save_algorithm(args):
    save_algorithm(args.path, args.name, args.desc)


def _kill(args):
    kill()


def main():
    parser = argparse.ArgumentParser(prog="khelper")
    sub = parser.add_subparsers(title="khelper sub command")

    k = sub.add_parser("kill")
    k.set_defaults(func=_kill)

    save = sub.add_parser("save")
    save_sub = save.add_subparsers(title="Content")

    parser.set_defaults(func=lambda x: parser.print_help())
    save.set_defaults(func=lambda x: save.print_help())

    d = save_sub.add_parser("dataset")
    d.set_defaults(func=_save_dataset)
    d.add_argument("--name", "-n", help="Dataset name", required=True)
    d.add_argument("--desc", "-d", help="Description for the dataset", required=False)
    d.add_argument("--path", "-p", help="Path to dataset", required=True)
    d.add_argument("--as_serialized", "-s", action='store_true', help='upload as kpl-dataset format dataset')

    p = save_sub.add_parser("model")
    p.set_defaults(func=_save_model)
    p.add_argument("--name", "-n", help="Model name", required=True)
    p.add_argument("--desc", "-d", help="Description for the model", required=False)
    p.add_argument("--path", "-p", help="Path to model", required=True)

    p = save_sub.add_parser("algorithm")
    p.set_defaults(func=_save_algorithm)
    p.add_argument("--name", "-n", help="Algorithm name", required=True)
    p.add_argument("--desc", "-d", help="Description for the algorithm", required=False)
    p.add_argument("--path", "-p", help="Path to algorithm code direction", required=True)

    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main()



PLOT_LINE = 'line'


# Deprecated
class MetricFigure:
    def __init__(self, title, y1_name, y1_type=PLOT_LINE, y2_name=None, y2_type=None):
        pass

    def push_metric(self, x, y1, y2=None):
        pass




from kpl_helper.base import get_config, _APIBase


class StopTask(_APIBase):
    def stop(self):
        if not get_config().get_inner():
            return
        self.post("/task/stop", json={"task_id": get_config().get_task_id()})


def kill():
    StopTask().stop()



from kpl_helper.parameter import (
    get_parameter,
    write_parameter,
)

from kpl_helper import metric, result, progress, io, deploy
from kpl_helper.base import ready, done
from kpl_helper.save import save_dataset, save_model, save_algorithm
from kpl_helper.progress import send_progress
from kpl_helper.stop import kill
from kpl_helper.io import get_input_path, get_output_path



from kpl_helper.base import logger, get_config, _APIBase


class Metric(_APIBase):
    def send_progress(self, progress):
        if not get_config().get_inner():
            return
        if not (0.0 <= progress <= 1.0):
            logger.warning("Progress should be in range [0.0, 1.0]. but get `{}`".format(progress))
            if progress < 0.0:
                progress = 0.0
            if progress > 1.0:
                progress = 1.0
        self.post("/task/attribute/update", json={
                                "task_id": get_config().get_task_id(),
                                "type": 'progress',
                                "name": 'progress',
                                "value": progress
                            })


def send_progress(progress):
    Metric().send_progress(progress)



import requests
from requests.adapters import HTTPAdapter
import logging
import sys
import os

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger("kpl-helper")

session = requests.Session()
session.mount('http://', HTTPAdapter(max_retries=3))
session.mount('https://', HTTPAdapter(max_retries=3))


class _BaseConfig:
    def __init__(self):
        _inner = os.getenv("KPL_INNER")
        self._inner = False if _inner is None else True
        if not self._inner:
            return
        # KPL_ENV: notebook/pipeline/deploy
        self._env_type = os.getenv("KPL_ENV")
        self._input_root = os.getenv("KPL_ENV_INPUT_ROOT_PATH")
        self._output_root = os.getenv("KPL_ENV_OUTPUT_ROOT_PATH", "/")
        self._parameter = os.getenv("KPL_ENV_PARAMETER")
        self._task_id = int(os.getenv("KPL_ENV_TASK_ID", 0))
        # TODO: remove  KPL_METRIC_TOKEN, KPL_METRIC_API
        self._metric_api = os.getenv("KPL_METRIC_API")
        self._metric_token = os.getenv("KPL_METRIC_TOKEN")
        self._api_url = os.getenv("KPL_ENV_INTERNAL_API_HOST", "http://seetaas--monitor.seetaas.svc.cluster.local:8920")
        if not self._api_url.endswith("/"):
            self._api_url = self._api_url + "/"
        jwt_token_path = os.getenv("KPL_TOKEN_LOCATE")
        with open(jwt_token_path) as fi:
            self._jwt_token = fi.read().strip()

    def get_api_url(self):
        if not self._inner:
            return ""
        if self._api_url.endswith("/"):
            return self._api_url[:len(self._api_url) - 1]
        return self._api_url

    def get_jwt_token(self):
        if not self._inner:
            return ""
        return self._jwt_token

    def get_inner(self):
        return self._inner

    def get_input_root(self):
        if not self._inner:
            return ""
        return self._input_root

    def get_output_root(self):
        if not self._inner:
            return ""
        return self._output_root

    def get_parameter(self):
        if not self._inner:
            return "{}"
        return self._parameter

    def get_metric_api(self):
        if not self._inner:
            return ""
        return self._metric_api

    def get_metric_token(self):
        if not self._inner:
            return ""
        return self._metric_token

    def get_env_type(self):
        if not self._inner:
            return None
        return self._env_type

    def get_task_id(self):
        if not self._inner:
            return None
        return self._task_id

    def get_cluster(self):
        pass


__base_config = None


def get_config():
    global __base_config
    if __base_config is None:
        __base_config = _BaseConfig()
    return __base_config


def ready():
    logging.info("[kpl-helper]: ready for using.")


def done():
    logging.info("[kpl-helper]: execute kpl done()")


class _MsgType:
    NewMetric = "NewMetric"
    MetricData = "MetricData"


class _ResultType:
    SCALAR_RESULT = 'scalar_result'  # 用于如Rank1，Rank5，LFW，MegaFace测试协议输出的单精度值测试结果
    CURVE_RESULT = 'curve_result'  # 用于如ROC测试协议输出的测试曲线
    PROGRESS = 'progress'


class _APIBase:
    def __init__(self):
        self._api = get_config().get_api_url()
        self._token = get_config().get_jwt_token()
        self.sess = requests.session()
        self.post = self._wrap(self.sess.post)

    def _wrap(self, func):
        def wrapped_http(router, **kwargs):
            if "headers" not in kwargs:
                kwargs["headers"] = {"Authorization": self._token}
            else:
                kwargs["headers"]["Authorization"] = self._token
            kwargs["url"] = self._api + router
            res = func(**kwargs)
            if not res.ok:
                raise Exception("Network error. status code:", res.status_code)
            response = res.json()
            if response['code'] != 'Success':
                raise Exception("Response error. code: [{}]. message: [{}]".format(response['code'], response['msg']))
            return response.get('data', None)

        return wrapped_http
